/* du32 v0.8 - small program to find files and directories size
 *
 * Copyright (C) 2004 Giuseppe Caulo <giusc@softzone.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>


#ifdef _WIN32
# include <direct.h>
# include "readdir.h"
# define snprintf _snprintf
# define strcasecmp stricmp
# ifndef __STDC__
#  define stat _stat
#  define lstat _stat
#  define fstat _fstat
#  define S_IFMT   _S_IFMT
#  define S_IFDIR  _S_IFDIR
#  define S_IFCHR  _S_IFCHR
#  define S_IFREG  _S_IFREG
#  define S_IREAD  _S_IREAD
#  define S_IWRITE _S_IWRITE
#  define S_IEXEC  _S_IEXEC
# endif
# define S_ISREG(x)  ((x) & _S_IFREG)
# define S_ISDIR(x)  ((x) & _S_IFDIR)
#else
#include <dirent.h>
#include <unistd.h>
#endif
#ifndef MAX_PATH
#define MAX_PATH 256
#endif
#ifndef PATH_MAX
#define PATH_MAX MAX_PATH
#endif

/* typedef unsigned long off_t; */

off_t get_du(const char *dir_name)
{
  DIR *dirp;
  struct dirent *dp;
  struct stat statbuf;
  off_t file_size = 0;

  if (dir_name == NULL)
    return 0;

  if (chdir(dir_name) == -1)
    return 0;

  if ((dirp = opendir(".")) == NULL)
    return 0;

  while ((dp=readdir(dirp)) != NULL) {
    if (!strcmp(dp->d_name, "..") || !strcmp(dp->d_name, "."))
      continue;
    if (lstat(dp->d_name,&statbuf)==0) {
	  if (statbuf.st_mode & S_IFDIR) {
        file_size += get_du(dp->d_name);
      } else if (S_ISREG(statbuf.st_mode)) {
	    file_size += statbuf.st_size;
	  }
    }
  }
  closedir(dirp);
  if (dir_name != NULL && strcmp(dir_name, ".." ) && strcmp(dir_name, "." ))
    chdir("..");
  return(file_size);
}

void scan (char *nome_dir) {
	DIR *directory = NULL;
	char *path = "";
	struct dirent *dir = NULL;
	struct stat data;

	if ((directory=opendir(nome_dir))!=NULL) {
	    while((dir=readdir(directory))) {
			if (!strncmp(dir->d_name, ".", 1) ||
					!strncmp(dir->d_name, "..", 2))
				continue;
			if((path=(char *)malloc(strlen(nome_dir)+strlen(dir->d_name)+2))==NULL) {
				fprintf(stdout,"Errore %s\n",strerror(errno));
				exit(-1);
			}
			sprintf(path,"%s/%s",nome_dir,dir->d_name);
			lstat(path,&data);
			if (S_ISDIR(data.st_mode))
				scan(path);
			printf("%s\n", path);
			if (path)
				free(path);
		}
		closedir(directory);
	} else {
		perror("scan()_opendir()");
		exit(-1);
	}
}

void usage(void) {
	    fprintf(stderr, "du32 v0.8\nWritten by Giuseppe Caulo\n\n");
		fprintf(stderr, "Usage: du32 [-a|-b|-k|-m|-g] <path>\n"
		                "-a     auto format output (default)\n"
		                "-b     format output with bytes\n"
		                "-k     format output with kilobytes\n"
		                "-m     format output with megabytes\n"
		                "-g     format output with gigabytes\n"
		                "\n"
		                "path   path to the directory\n\n");
		exit(-1);
}

char *format_size(unsigned long bytes, char type) {
	unsigned long divfactor;
	char *retfmt = NULL;
#define RETSIZE 128
	switch (type) {
		case 'b':
		  divfactor = 1;
		  break;
		case 'k':
		  divfactor = 1024;
		  break;
		case 'm':
		  divfactor = 1024 * 1024;
		  break;
		case 'g':
		  divfactor = 1024 * 1024 * 1024;
		  break;
		default:
		  divfactor = 0;
	}
	retfmt = (char *)malloc(RETSIZE+1);
	if (!retfmt) return NULL;
	if (divfactor) {
	  snprintf(retfmt, RETSIZE, "%lu", (bytes / divfactor));
	} else {
	  float kb = (float)(bytes / 1024.00);
	  if (bytes < 1000) {
		  snprintf(retfmt, RETSIZE, " %d", (int)bytes);
	  } else if (bytes < 1024) {
		  snprintf(retfmt, RETSIZE, "%0.2fk", kb);
	  } else if (kb < 10) {
		  snprintf(retfmt, RETSIZE, "%1.1fk", kb);
	  } else {
		  float mb = (float)(kb / 1024.00);
		  if (kb < 1000) {
			  snprintf(retfmt, RETSIZE, "%3.0fk", kb);
		  } else if (kb < 1024) {
			  snprintf(retfmt, RETSIZE, "%0.2fM", mb);
		  } else if (mb < 10) {
			  snprintf(retfmt, RETSIZE, "%1.1fM", mb);
		  } else {
			  float gb = (float)(mb / 1024.00);
			  if (mb < 1000) {
				  snprintf(retfmt, RETSIZE, "%3.0fM", mb);
			  } else if (mb < 1024) {
				  snprintf(retfmt, RETSIZE, "%0.2fG", gb);
			  } else if (gb  < 10) {
				  snprintf(retfmt, RETSIZE, "%1.1fG", gb);
			  } else {
				  snprintf(retfmt, RETSIZE, "%3.0fG", gb);
			  }
		  }
	  }
	}
	return retfmt;
}

int main (int argc, char **argv) {
	struct stat statbuf;
	char *path = NULL;
	int i;
	char fmt = 0;
	if (argc == 2)
	  path = argv[1];
	else if (argc == 3)
	{
		int status = 0;
		for (i = 1; i < argc; i++) {
			if (argv[i][0] != '-') {
				if (!path)
				  path = argv[i];
				else {
				  status = 1;
				  break;
				}
			} else {
				fmt = argv[i][1];
			}
		}
		if (status || !path)
		  usage();
	} else {
		usage();
	}
	i = strlen(path);
	if ((i > 2) && (path[i - 2] != ':') && ((path[i - 1] == '\\') || (path[i - 1] == '/')))
	  path[i - 1] = '\0';
	lstat(path, &statbuf);
	if (S_ISDIR(statbuf.st_mode)) {
		off_t bytes = get_du(path);
	  char *sz = format_size(bytes, fmt);
      printf("%s\n", sz);
      if (sz) free(sz);
      return 0;
    } else {
	  fprintf(stderr, "du32: '%s' not a regular directory\n", argv[1]);
	  return -1;
    }
}



